Attribute VB_Name = "modListView"
' Modulname : vbzListView.bas
' Version   : 2.50
' Date      : 2018-12-27
' Source    : http://www.vb-zentrum.de
' Author    : ralf schlegel

' IMPORTAND NOTES:
' for WinXP and above; needs Microsoft Windows Common Controls 5.0(SP2) - comctl32.ocx,
' because many functions, such as "lvwSetExtendedStyle" are NOT supported in version 6.0
'
' Make sure your project takes a reference to "OLE Automation" (stdole2.tlb)
' this reference it needed for the 'IPicture' object!
'
' HISTORY (DD-MM-YYYY):
' 2011-03-04: expand LVW_EX_STYLES for Vista/Win7
'             fixed up "lvwSetBackgroundImage" (clear WATERMARK)
'             "lvwSubItemHitTest" fixed, documentation of subitem value
' 2011-05-21: added "lvwSetImageList" shared mode
'             added "lvwRedrawItems"
' 2011-07-28: "lvwSortColumn" added "FILEDATE" parameter, for sorting by datetime
'             with sections of folders and files
' 2011-08-02: added ListView Notification LVN_ constants
' 2011-08-12: "lvwGetUnicodeFormat" added
'             "lvwSetUnicodeFormat" updated
' 2012-10-13: optimized unicode functions
' 2013-01-31: bugfix "lvwSelectAll"
' 2013-02-17: added Common control shared messages (CCM_* )
' 2013-04-27: added "lvwSetItemIcon" to set an icon to a subItem in report view (see function remarks)!
' 2014-02-12: added "lvwDeselectAll" to quick remove all selections
' 2014-04-05: added "lvwRemoveItemState"
' 2015-08-15: added "lvwGetItemRect"
' 2016-08-21: bugfix "lvwSelItemsToArray" (correction of returned array)
' 2017-08-30: added "lvwGetItemPosition", optimized parameters (ByVal / ByRef)
' 2018-04-22: added "lvwSetOverlayImage", see function for documentation
' 2018-11-15: added "lvwSetItemStateMask" to en-/disable ItemState callbacks
' 2018-12-27: added "lvwHeader..." functions (arrow for sort order and more)
Option Explicit

Private Const CCM_FIRST = &H2000                       ' Common control shared messages
Private Const CCM_SETUNICODEFORMAT = (CCM_FIRST + 5)
Private Const CCM_GETUNICODEFORMAT = (CCM_FIRST + 6)

' ListView Message Constants:
Public Const LVM_FIRST As Long = &H1000&
Public Const LVM_GETBKCOLOR As Long = (LVM_FIRST + 0)
Public Const LVM_SETBKCOLOR As Long = (LVM_FIRST + 1)
Public Const LVM_GETIMAGELIST As Long = (LVM_FIRST + 2)
Public Const LVM_SETIMAGELIST As Long = (LVM_FIRST + 3)
Public Const LVM_GETITEMCOUNT As Long = (LVM_FIRST + 4)
Public Const LVM_GETITEM As Long = (LVM_FIRST + 5)
Public Const LVM_SETITEM As Long = (LVM_FIRST + 6)
Public Const LVM_INSERTITEM As Long = (LVM_FIRST + 7)
Public Const LVM_DELETEITEM As Long = (LVM_FIRST + 8)
Public Const LVM_DELETEALLITEMS As Long = (LVM_FIRST + 9)
Public Const LVM_GETCALLBACKMASK As Long = (LVM_FIRST + 10)
Public Const LVM_SETCALLBACKMASK As Long = (LVM_FIRST + 11)
Public Const LVM_GETNEXTITEM As Long = (LVM_FIRST + 12)
Public Const LVM_FINDITEMA  As Long = (LVM_FIRST + 13)
Public Const LVM_GETITEMRECT As Long = (LVM_FIRST + 14)
Public Const LVM_SETITEMPOSITION As Long = (LVM_FIRST + 15)
Public Const LVM_GETITEMPOSITION As Long = (LVM_FIRST + 16)
Public Const LVM_GETSTRINGWIDTH As Long = (LVM_FIRST + 17)
Public Const LVM_HITTEST As Long = (LVM_FIRST + 18)
Public Const LVM_ENSUREVISIBLE As Long = (LVM_FIRST + 19)
Public Const LVM_SCROLL As Long = (LVM_FIRST + 20)
Public Const LVM_REDRAWITEMS As Long = (LVM_FIRST + 21)
Public Const LVM_ARRANGE As Long = (LVM_FIRST + 22)
Public Const LVM_EDITLABEL As Long = (LVM_FIRST + 23)
Public Const LVM_GETEDITCONTROL As Long = (LVM_FIRST + 24)
Public Const LVM_GETCOLUMN As Long = (LVM_FIRST + 25)
Public Const LVM_SETCOLUMN  As Long = (LVM_FIRST + 26)
Public Const LVM_INSERTCOLUMNA As Long = (LVM_FIRST + 27)
Public Const LVM_DELETECOLUMN As Long = (LVM_FIRST + 28)
Public Const LVM_GETCOLUMNWIDTH As Long = (LVM_FIRST + 29)
Public Const LVM_SETCOLUMNWIDTH As Long = (LVM_FIRST + 30)
Public Const LVM_GETHEADER As Long = (LVM_FIRST + 31)

Public Const LVM_CREATEDRAGIMAGE As Long = (LVM_FIRST + 33)
Public Const LVM_GETVIEWRECT As Long = (LVM_FIRST + 34)
Public Const LVM_GETTEXTCOLOR As Long = (LVM_FIRST + 35)
Public Const LVM_SETTEXTCOLOR As Long = (LVM_FIRST + 36)
Public Const LVM_GETTEXTBKCOLOR As Long = (LVM_FIRST + 37)
Public Const LVM_SETTEXTBKCOLOR As Long = (LVM_FIRST + 38)
Public Const LVM_GETTOPINDEX As Long = (LVM_FIRST + 39)
Public Const LVM_GETCOUNTPERPAGE As Long = (LVM_FIRST + 40)
Public Const LVM_GETORIGIN As Long = (LVM_FIRST + 41)
Public Const LVM_UPDATE  As Long = (LVM_FIRST + 42)
Public Const LVM_SETITEMSTATE As Long = (LVM_FIRST + 43)
Public Const LVM_GETITEMSTATE As Long = (LVM_FIRST + 44)
Public Const LVM_GETITEMTEXTA As Long = (LVM_FIRST + 45)
Public Const LVM_SETITEMTEXTA As Long = (LVM_FIRST + 46)
Public Const LVM_SETITEMCOUNT As Long = (LVM_FIRST + 47)
Public Const LVM_SORTITEMS As Long = (LVM_FIRST + 48)
Public Const LVM_SETITEMPOSITION32 As Long = (LVM_FIRST + 49)
Public Const LVM_GETSELECTEDCOUNT As Long = (LVM_FIRST + 50)
Public Const LVM_GETITEMSPACING As Long = (LVM_FIRST + 51)
Public Const LVM_GETISEARCHSTRING As Long = (LVM_FIRST + 52)
Public Const LVM_SETICONSPACING As Long = (LVM_FIRST + 53)
Public Const LVM_SETEXTENDEDLISTVIEWSTYLE As Long = (LVM_FIRST + 54)
Public Const LVM_GETEXTENDEDLISTVIEWSTYLE As Long = (LVM_FIRST + 55)
Public Const LVM_GETSUBITEMRECT As Long = (LVM_FIRST + 56)
Public Const LVM_SUBITEMHITTEST As Long = (LVM_FIRST + 57)
Public Const LVM_SETCOLUMNORDERARRAY As Long = (LVM_FIRST + 58)
Public Const LVM_GETCOLUMNORDERARRAY As Long = (LVM_FIRST + 59)
Public Const LVM_SETHOTITEM As Long = (LVM_FIRST + 60)
Public Const LVM_GETHOTITEM As Long = (LVM_FIRST + 61)
Public Const LVM_SETHOTCURSOR As Long = (LVM_FIRST + 62)
Public Const LVM_GETHOTCURSOR As Long = (LVM_FIRST + 63)
Public Const LVM_APPROXIMATEVIEWRECT As Long = (LVM_FIRST + 64)
Public Const LVM_SETWORKAREAS As Long = (LVM_FIRST + 65)
Public Const LVM_GETSELECTIONMARK As Long = (LVM_FIRST + 66)
Public Const LVM_SETSELECTIONMARK As Long = (LVM_FIRST + 67)
Public Const LVM_SETBKIMAGE As Long = (LVM_FIRST + 68)
Public Const LVM_GETBKIMAGE As Long = (LVM_FIRST + 69)
Public Const LVM_GETWORKAREAS As Long = (LVM_FIRST + 70)
Public Const LVM_SETHOVERTIME As Long = (LVM_FIRST + 71)
Public Const LVM_GETHOVERTIME As Long = (LVM_FIRST + 72)
Public Const LVM_GETNUMBEROFWORKAREAS As Long = (LVM_FIRST + 73)
Public Const LVM_SETTOOLTIPS As Long = (LVM_FIRST + 74)
Public Const LVM_GETTOOLTIPS As Long = (LVM_FIRST + 78)
Public Const LVM_SORTITEMSEX As Long = (LVM_FIRST + 81)
' UNICODE Section:
Public Const LVM_INSERTITEMW As Long = (LVM_FIRST + 77)
Public Const LVM_FINDITEMW As Long = (LVM_FIRST + 83)
Public Const LVM_GETITEMTEXTW As Long = (LVM_FIRST + 115)
Public Const LVM_SETITEMTEXTW As Long = (LVM_FIRST + 116)

' ListView Header Message Constants:
Private Const HDM_FIRST = &H1200
Private Const HDM_GETITEMCOUNT = (HDM_FIRST + 0)
Private Const HDM_INSERTITEMA = (HDM_FIRST + 1)
Private Const HDM_DELETEITEM = (HDM_FIRST + 2)
Private Const HDM_GETITEMA = (HDM_FIRST + 3)
Private Const HDM_SETITEMA = (HDM_FIRST + 4)
Private Const HDM_LAYOUT = (HDM_FIRST + 5)
Private Const HDM_GETITEMRECT = (HDM_FIRST + 7)
Private Const HDM_SETIMAGELIST = (HDM_FIRST + 8)
Private Const HDM_GETIMAGELIST = (HDM_FIRST + 9)
Private Const HDM_INSERTITEMW = (HDM_FIRST + 10)
Private Const HDM_GETITEMW = (HDM_FIRST + 11)
Private Const HDM_SETITEMW = (HDM_FIRST + 12)
Private Const HDM_ORDERTOINDEX = (HDM_FIRST + 15)


'LVM_GETITEMRECT Messages (LVIR_)
Public Enum LVIR_FLAG
  LVIR_BOUNDS = 0         ' complete rectangle (full raw)
  LVIR_ICON = 1           ' icon rectangle
  LVIR_LABEL = 2          ' text rectangle
  LVIR_SELECTBOUNDS = 3   ' selection rectangle
End Enum
#If False Then
  Const LVIR_BOUNDS = 0
  Const LVIR_ICON = 1
  Const LVIR_LABEL = 2
  Const LVIR_SELECTBOUNDS = 3
#End If

' ListView Notification constants (LVN_) for subclassing:
Public Const LVN_FIRST            As Long = (-100)
Public Const LVN_LAST             As Long = (-199)
Public Const LVN_BEGINDRAG        As Long = (LVN_FIRST - 9)
Public Const LVN_BEGINLABELEDITA  As Long = (LVN_FIRST - 5)
Public Const LVN_BEGINLABELEDITW  As Long = (LVN_FIRST - 75)
Public Const LVN_BEGINRDRAG       As Long = (LVN_FIRST - 11)
Public Const LVN_BEGINSCROLL      As Long = (LVN_FIRST - 80)
Public Const LVN_COLUMNCLICK      As Long = (LVN_FIRST - 8)
Public Const LVN_DELETEALLITEMS   As Long = (LVN_FIRST - 4)
Public Const LVN_DELETEITEM       As Long = (LVN_FIRST - 3)
Public Const LVN_ENDLABELEDITA    As Long = (LVN_FIRST - 6)
Public Const LVN_ENDLABELEDITW    As Long = (LVN_FIRST - 76)
Public Const LVN_ENDSCROLL        As Long = (LVN_FIRST - 81)
Public Const LVN_GETDISPINFOA     As Long = (LVN_FIRST - 50)
Public Const LVN_GETDISPINFOW     As Long = (LVN_FIRST - 77)
Public Const LVN_GETINFOTIPA      As Long = (LVN_FIRST - 57)
Public Const LVN_GETINFOTIPW      As Long = (LVN_FIRST - 58)
Public Const LVN_HOTTRACK         As Long = (LVN_FIRST - 21)
Public Const LVN_INSERTITEM       As Long = (LVN_FIRST - 2)
Public Const LVN_ITEMACTIVATE     As Long = (LVN_FIRST - 14)
Public Const LVN_ITEMCHANGED      As Long = (LVN_FIRST - 1)
Public Const LVN_ITEMCHANGING     As Long = (LVN_FIRST - 0)
Public Const LVN_KEYDOWN          As Long = (LVN_FIRST - 55)
Public Const LVN_MARQUEEBEGIN     As Long = (LVN_FIRST - 56)
Public Const LVN_ODCACHEHINT      As Long = (LVN_FIRST - 13)
Public Const LVN_ODFINDITEMA      As Long = (LVN_FIRST - 52)
Public Const LVN_ODFINDITEMW      As Long = (LVN_FIRST - 79)
Public Const LVN_ODSTATECHANGED   As Long = (LVN_FIRST - 15)
Public Const LVN_SETDISPINFOA     As Long = (LVN_FIRST - 51)
Public Const LVN_SETDISPINFOW     As Long = (LVN_FIRST - 78)

' LVM_SET/GETBKIMAGE Flags
Private Const LVBKIF_SOURCE_NONE As Long = &H0            ' Delete background image or watermark
Private Const LVBKIF_SOURCE_HBITMAP As Long = &H1         ' WinXP or higher, only visible in compiled mode
Private Const LVBKIF_SOURCE_URL As Long = &H2             ' Image is a file
Private Const LVBKIF_SOURCE_MASK As Long = &H3
Private Const LVBKIF_STYLE_NORMAL As Long = &H0
Private Const LVBKIF_STYLE_TILE As Long = &H10
Private Const LVBKIF_FLAG_TILEOFFSET As Long = &H100
Private Const LVBKIF_TYPE_WATERMARK As Long = &H10000000  ' WinXP or higher, only visible in compiled mode
Private Const LVBKIF_FLAG_ALPHABLEND As Long = &H20000000 ' WinXP or higher, only visible in compiled mode

Private Type POINTAPI
  X As Long
  Y As Long
End Type

Private Type LVFINDINFOW
  Flags As Long
  psz As Long
  lParam As Long
  pt As POINTAPI
  vkDirection As Long
End Type
Private Type LVFINDINFOA
  Flags As Long
  psz As String
  lParam As Long
  pt As POINTAPI
  vkDirection As Long
End Type

Public Enum LVA_MODES
  LVA_DEFAULT = &H0
  LVA_ALIGNLEFT = &H1
  LVA_ALIGNTOP = &H2
  LVA_SNAPTOGRID = &H5
End Enum
#If False Then
  Const LVA_DEFAULT = 0
  Const LVA_ALIGNLEFT = 1
  Const LVA_ALIGNTOP = 2
  Const LVA_SNAPTOGRID = 3
#End If

' ListView styles
Public Enum LVW_STYLES
  LVS_ICON = &H0
  LVS_REPORT = &H1
  LVS_SMALLICON = &H2
  LVS_LIST = &H3
  LVS_TYPEMASK = &H3
  
  LVS_SINGLESEL = &H4
  LVS_SHOWSELALWAYS = &H8
  LVS_SORTASCENDING = &H10
  LVS_SORTDESCENDING = &H20
  LVS_SHAREIMAGELISTS = &H40
  LVS_NOLABELWRAP = &H80
  LVS_AUTOARRANGE = &H100
  LVS_EDITLABELS = &H200
  LVS_OWNERDRAWFIXED = &H400
  LVS_OWNERDATA = &H1000
  LVS_NOSCROLL = &H2000
  LVS_NOCOLUMNHEADER = &H4000
  LVS_NOSORTHEADER = &H8000&
 
  LVS_ALIGNTOP = &H0
  LVS_ALIGNLEFT = &H800
  LVS_ALIGNMASK = &HC00
  LVS_TYPESTYLEMASK = &HFC00
End Enum   ' LVW_STYLES

' ListView extended styles
Public Enum LVW_EX_STYLES
  LVS_EX_NONE = &H0&
  LVS_EX_GRIDLINES = &H1&
  LVS_EX_SUBITEMIMAGES = &H2&
  LVS_EX_CHECKBOXES = &H4&
  LVS_EX_TRACKSELECT = &H8&
  LVS_EX_HEADERDRAGDROP = &H10&
  LVS_EX_FULLROWSELECT = &H20&
  LVS_EX_ONECLICKACTIVATE = &H40&
  LVS_EX_TWOCLICKACTIVATE = &H80&
  LVS_EX_FLATSB = &H100&
  LVS_EX_REGIONAL = &H200&
  LVS_EX_INFOTIP = &H400&
  LVS_EX_UNDERLINEHOT = &H800&
  LVS_EX_UNDERLINECOLD = &H1000&
  LVS_EX_MULTIWORKAREAS = &H2000&
  LVS_EX_LABELTIP = &H4000&
  LVS_EX_BORDERSELECT = &H8000&
  LVS_EX_DOUBLEBUFFER = &H10000
  LVS_EX_HIDELABELS = &H20000
  LVS_EX_SINGLEROW = &H40000              ' not used (see MSN docu)
  LVS_EX_SNAPTOGRID = &H80000             ' in icon view, icons automatically snap into grid
  LVS_EX_SIMPLESELECT = &H100000
  ' new Styles for MS VISTA and higher:
  LVS_EX_JUSTIFYCOLUMNS = &H200000        ' icons are lined up in columns that use up the whole view
  LVS_EX_TRANSPARENTBKGND = &H400000      ' background is painted by the parent via
  LVS_EX_TRANSPARENTSHADOWTEXT = &H800000 ' enable shadow text on transparent backgrounds only
  LVS_EX_AUTOAUTOARRANGE = &H1000000      ' similar to LVS_AUTOARRANGE
  LVS_EX_HEADERINALLVIEWS = &H2000000     ' show header in all views!
  LVS_EX_AUTOCHECKSELECT = &H8000000      ' automatically select check boxes on single click
  LVS_EX_AUTOSIZECOLUMNS = &H10000000     ' automatically size listview columns
  LVS_EX_COLUMNSNAPPOINTS = &H40000000    ' snap to min. column width when user resizes a column
  LVS_EX_COLUMNOVERFLOW = &H80000000      ' needs also LVS_EX_HEADERINALLVIEWS (see MS documentation)
End Enum   ' LVW_EX_STYLES

' ListView Item State (LVIS)
Public Enum LVW_ITEM_STATE
  LVIS_FOCUSED = &H1&
  LVIS_SELECTED = &H2&
  LVIS_CUT = &H4&
  LVIS_DROPHILITED = &H8&
  LVIS_OVERLAYMASK = &HF00&
  LVIS_UNCHECKED = &H1000&
  LVIS_CHECKED = &H2000&
  LVIS_STATEIMAGEMASK = &HF000&
End Enum
#If False Then
  Const LVIS_FOCUSED = &H1&
  Const LVIS_SELECTED = &H2&
  Const LVIS_CUT = &H4&
  Const LVIS_DROPHILITED = &H8&
  Const LVIS_OVERLAYMASK = &HF00&
  Const LVIS_UNCHECKED = &H1000&
  Const LVIS_CHECKED = &H2000&
  Const LVIS_STATEIMAGEMASK = &HF000&
#End If

Public Enum LVSIL_Flags
  LVSIL_NORMAL = 0
  LVSIL_SMALL = 1
  LVSIL_STATE = 2
  LVSIL_GROUPHEADER = 3
End Enum
#If False Then
  Const LVSIL_NORMAL = 0
  Const LVSIL_SMALL = 1
  Const LVSIL_STATE = 2
  Const LVSIL_GROUPHEADER = 3
#End If

' ListView Index Find (LVIF)
Private Const LVIF_TEXT As Long = &H1&
Private Const LVIF_IMAGE As Long = &H2&
Private Const LVIF_STATE As Long = &H8&

' ListView Find Item (LVFI)
Private Const LVFI_PARAM As Long = &H1&
Private Const LVFI_STRING As Long = &H2&
Private Const LVFI_SUBSTRING As Long = &H4& ' MS Vista and later!
Private Const LVFI_PARTIAL As Long = &H8&
Private Const LVFI_WRAP As Long = &H20&
Private Const LVFI_NEARESTXY As Long = &H40&

' ListView Header Constants
' HDITEM mask
Private Const HDI_WIDTH = &H1
Private Const HDI_HEIGHT = &H1
Private Const HDI_TEXT = &H2
Private Const HDI_FORMAT = &H4
Private Const HDI_LPARAM = &H8
Private Const HDI_BITMAP = &H10
Private Const HDI_IMAGE = &H20
Private Const HDI_DI_SETITEM = &H40
Private Const HDI_ORDER = &H80
Private Const HDI_FILTER = &H100

' HDITEM fmt
Private Const HDF_LEFT = &H0
Private Const HDF_RIGHT = &H1
Private Const HDF_CENTER = &H2

Private Const HDF_SORTDOWN = &H200
Private Const HDF_SORTUP = &H400
Private Const HDF_IMAGE = &H800
Private Const HDF_BITMAP_ON_RIGHT = &H1000  ' show bitmap on right side of text
Private Const HDF_BITMAP = &H2000           ' show bitmap on left side (default)
Private Const HDF_STRING = &H4000


Private Const HDS_BUTTONS As Long = &H2&
Private Const WM_VSCROLL As Long = &H115&

Private Const GWL_EXSTYLE As Long = -20
Private Const GWL_HINSTANCE As Long = -6
Private Const GWL_HWNDPARENT As Long = -8
Private Const GWL_ID As Long = -12
Private Const GWL_STYLE As Long = -16
Private Const GWL_USERDATA As Long = -21
Private Const GWL_WNDPROC As Long = -4
Private Const SWP_DRAWFRAME As Long = &H20
Private Const SWP_NOMOVE As Long = &H2
Private Const SWP_NOSIZE As Long = &H1
Private Const SWP_NOZORDER As Long = &H4
Private Const SWP_FLAGS As Long = SWP_NOZORDER Or SWP_NOSIZE Or SWP_NOMOVE Or SWP_DRAWFRAME
Private Const WS_CLIPCHILDREN As Long = &H2000000

' Typendefinitionen:
Private Type HDITEMA  ' HD_ITEM ANSI
  mask As Long
  cxy As Long
  pszText As String   ' if retrieving text, must be pre-allocated
  hBM As Long
  cchTextMax As Long
  fmt As Long
  lParam As Long
  iImage As Long      ' index of bitmap in ImageList
  iOrder As Long      ' where to draw this item
End Type
Private Type HDITEMW  ' HD_ITEM UNICODE
  mask As Long
  cxy As Long
  pszText As Long     ' if retrieving text, must be pre-allocated
  hBM As Long
  cchTextMax As Long
  fmt As Long
  lParam As Long
  iImage As Long      ' index of bitmap in ImageList
  iOrder As Long      ' where to draw this item
End Type

Public Type LVCOLUMN
  mask As Long
  fmt As Long
  cx As Long
  pszText As String
  cchTextMax As Long
  iSubItem As Long
  iOrder As Long
  iImage As Long
End Type
Public Type LVCOLUMNW
  mask As Long
  fmt As Long
  cx As Long
  pszText As Long
  cchTextMax As Long
  iSubItem As Long
  iOrder As Long
  iImage As Long
End Type

Public Type LVITEM
  mask As Long
  IItem As Long
  iSubItem As Long
  State As LVW_ITEM_STATE
  stateMask As LVW_ITEM_STATE
  pszText As String
  cchTextMax As Long
  iImage As Long
  lParam As Long
  iIndent As Long
' >= Comctl32.dll V5.01 (WinXP and newer)
  iGroupId As Long
  cColumns As Long
  puColumns As Long
' >= Comctl32.dll V6.0 (Vista and newer)
'  piColFmt As Long
'  iGroup As Long
End Type
Public Type LVITEMW
  mask As Long
  IItem As Long
  iSubItem As Long
  State As LVW_ITEM_STATE
  stateMask As LVW_ITEM_STATE
  pszText As Long
  cchTextMax As Long
  iImage As Long
  lParam As Long
  iIndent As Long
' >= Comctl32.dll V5.01 (WinXP and newer)
  iGroupId As Long
  cColumns As Long
  puColumns As Long
' >= Comctl32.dll V6.0 (Vista and newer)
'  piColFmt As Long
'  iGroup As Long
End Type

Public Type LVHITTESTINFO
  pt As POINTAPI
  Flags As LVHITTESTINFO_flags
  IItem As Long
  iSubItem As Long
End Type

Public Enum LVHITTESTINFO_flags
  LVHT_NOWHERE = &H1   ' in LV client area, but not over item
  LVHT_ONITEMICON = &H2
  LVHT_ONITEMLABEL = &H4
  LVHT_ONITEMSTATEICON = &H8
  LVHT_ONITEM = (LVHT_ONITEMICON Or LVHT_ONITEMLABEL Or LVHT_ONITEMSTATEICON)
  ' outside the LV's client area
  LVHT_ABOVE = &H8
  LVHT_BELOW = &H10
  LVHT_TORIGHT = &H20
  LVHT_TOLEFT = &H40
End Enum

Private Type LVBKIMAGE
  ulFlags As Long                       ' LVBKIF_*
  hBM As Long                           ' bitmap handle
  pszImage As String                    ' file name
  cchImageMax As Long                   ' size of max file name
  xOffsetPercent As Long                ' x offset
  yOffsetPercent As Long                ' y offset
End Type
Private Const CLR_DEFAULT = &HFF000000  ' default background color
Private Const CLR_NONE = &HFFFFFFFF     ' no background color (transparent)

' Declaration window and system functions:
Private Declare Function SendMessageA Lib "user32" (ByVal hWnd As Long, _
        ByVal wMsg As Long, ByVal wParam As Long, lParam As Any) As Long
Private Declare Function SendMessageW Lib "user32" (ByVal hWnd As Long, _
        ByVal wMsg As Long, ByVal wParam As Long, lParam As Any) As Long
Private Declare Function GetCursorPos Lib "user32" (lpPoint As POINTAPI) As Long
Private Declare Function ScreenToClient Lib "user32" (ByVal hWnd As Long, lpPoint As POINTAPI) As Long
Private Declare Function GetParent Lib "user32" (ByVal hWnd As Long) As Long
Private Declare Function GetWindowLongA Lib "user32" (ByVal hWnd As Long, ByVal nIndex As Long) As Long
Private Declare Function SetWindowLongA Lib "user32" (ByVal hWnd As Long, ByVal nIndex As Long, _
        ByVal dwNewLong As Long) As Long
Private Declare Function SetWindowPos Lib "user32" (ByVal hWnd As Long, ByVal hWndInsertAfter As Long, _
        ByVal X As Long, ByVal Y As Long, ByVal cx As Long, ByVal cy As Long, ByVal wFlags As Long) As Long
Private Declare Function EnableWindow Lib "user32" (ByVal hWnd As Long, ByVal fEnable As Long) As Long
Private Declare Function LockWindowUpdate Lib "user32" (ByVal hWnd As Long) As Long
Private Declare Function GetWindowRect Lib "user32" (ByVal hWnd As Long, lpRect As RECT) As Long

' Declarations for image list:
Private Declare Function ImageList_BeginDrag Lib "comctl32" (ByVal himlTrack As Long, _
        ByVal iTrack As Long, ByVal dxHotspot As Long, ByVal dyHotspot As Long) As Long
Private Declare Sub ImageList_EndDrag Lib "comctl32" ()
Private Declare Function ImageList_DragEnter Lib "comctl32" (ByVal hwndLock As Long, _
        ByVal X As Long, ByVal Y As Long) As Long
Private Declare Function ImageList_DragLeave Lib "comctl32" (ByVal hwndLock As Long) As Long
Private Declare Function ImageList_DragMove Lib "comctl32" (ByVal X As Long, ByVal Y As Long) As Long
Private Declare Function ImageList_Destroy Lib "comctl32" (ByVal himl As Long) As Long
Private Declare Function ImageList_GetImageCount Lib "comctl32" (ByVal himl As Long) As Long
Private Declare Function ImageList_SetOverlayImage Lib "comctl32" (ByVal himl As Long, _
        ByVal iImage As Long, ByVal iOverlay As Long) As Long
Private Const LVNI_SELECTED = &H2
Private Const LVNI_DROPHILITED = &H8

' ***  Private graphic functions (gdi32) for background and watermark effect:
Private Declare Function GetObjectA Lib "gdi32" (ByVal hObject As Long, ByVal nCount As Long, _
        ByRef lpObject As Any) As Long
Private Declare Function CreateDCA Lib "gdi32" (ByVal lpDriverName As String, _
        lpDeviceName As Any, lpOutput As Any, lpInitData As Any) As Long
Private Declare Function SelectObject Lib "gdi32" (ByVal hdc As Long, ByVal hObject As Long) As Long
Private Declare Function CreateCompatibleDC Lib "gdi32" (ByVal hdc As Long) As Long
Private Declare Function DeleteDC Lib "gdi32" (ByVal hdc As Long) As Long
Private Declare Function CreateCompatibleBitmap Lib "gdi32" (ByVal hdc As Long, _
        ByVal nWidth As Long, ByVal nHeight As Long) As Long
Private Declare Function BitBlt Lib "gdi32" (ByVal hDestDC As Long, _
        ByVal X As Long, ByVal Y As Long, ByVal nWidth As Long, ByVal nHeight As Long, _
        ByVal hSrcDC As Long, ByVal xSrc As Long, ByVal ySrc As Long, ByVal dwRop As Long) As Long
       
Private Type BITMAP
  bmType As Long
  bmWidth As Long
  bmHeight As Long
  bmWidthBytes As Long
  bmPlanes As Long
  bmBitsPixel As Integer
  bmBits As Long
End Type
' *** End graphic functions

' *** Begin XP themes and styles
Private Declare Function LoadLibraryW Lib "kernel32" (ByVal lpLibFileName As Long) As Long
Private Declare Function FreeLibrary Lib "kernel32" (ByVal hLibModule As Long) As Long
Private Declare Function SetWindowTheme Lib "uxtheme" (ByVal hWnd As Long, ByVal pSubAppName As Long, _
        ByVal pSubIDList As Long) As Long
Private Enum puxTHEME
  uxUNKNOWN = &H0&
  uxNONE = &H1&
  uxOKAY = &H2&
End Enum
Private m_uxtheme As puxTHEME
' *** End XP themes

Private lastSelItem As Long

' create an image of an ImageList as overlay image
' himl     = the ImageList handle
' iImage   = the zero based image number to be used as overlay image
' iOverlay = the overlay number form 1 to 15 to be used in function 'lvwSetOverlayImage'
Public Function imlCreateOverlayImage(ByVal himl As Long, ByVal iImage As Long, ByVal iOverlay As Long) As Boolean
  iOverlay = Abs(iOverlay - 1) Mod 15 + 1   ' make sure iOverlay is a value between 1 and 15!
  imlCreateOverlayImage = ImageList_SetOverlayImage(himl, iImage, iOverlay)
End Function

' initialize a Listview control with extended settings, instead of calling all these needed functions separately!
' returns the ListView's handle (put this to a global value inside your code, for easier further function calls)
' example
Public Function lvwAllInit(ByVal lvhWnd As Long, Optional ByVal addExStyles As LVW_EX_STYLES, _
                           Optional ByVal blExplorerStyle As Boolean, Optional ByVal disableStates _
                           As LVW_ITEM_STATE, Optional ByRef headerHandle As Long) As Long
  lvwSetUnicodeFormat lvhWnd, True                                        ' first set to unicode support!
  headerHandle = lvwHeaderGetHandle(lvhWnd)                               ' returns the header handle to your code for further functions
  If addExStyles Then lvwSetExtendedStyle lvhWnd, addExStyles, True       ' add extended styles
  If disableStates Then lvwSetItemStateMask lvhWnd, disableStates, False  ' remove not wanted states
  If blExplorerStyle Then lvwSetExplorerStyle lvhWnd                      ' add themed explorer style
  lvwAllInit = lvhWnd                                                     ' returns the list view's handle to your code
End Function

Public Function lvwArrangeItems(ByVal lvhWnd As Long, ByVal aMode As LVA_MODES) As Boolean
  lvwArrangeItems = SendMessageA(lvhWnd, LVM_ARRANGE, aMode, 0)
End Function

' return the found index from 1 to ListItem.Count, 0 means: nothing found!
' lvhWnd = handle of the listview
Public Function lvwFindItem(ByVal lvhWnd As Long, ByVal searchText As String) As Long
  If SendMessageA(lvhWnd, CCM_GETUNICODEFORMAT, 0, ByVal 0) Then
    Dim lvfiw As LVFINDINFOW
    lvfiw.Flags = LVFI_STRING
    lvfiw.psz = StrPtr(searchText)
    lvwFindItem = SendMessageW(lvhWnd, LVM_FINDITEMW, -1, lvfiw) + 1
  Else
    Dim lvfia As LVFINDINFOA
    lvfia.Flags = LVFI_STRING
    lvfia.psz = searchText
    lvwFindItem = SendMessageA(lvhWnd, LVM_FINDITEMA, -1, lvfia) + 1
  End If
End Function

' search for the first selected item and returns its index from 1 to ListItem.Count
' returns '0' if no item is seleted
Public Function lvwFindFirstSelItem(ByVal lvhWnd As Long) As Long
  lastSelItem = SendMessageA(lvhWnd, LVM_GETNEXTITEM, -1, ByVal LVIS_SELECTED)
  lvwFindFirstSelItem = lastSelItem + 1
End Function

' search for the next selected item and returns its index from 1 to ListItem.Count
' may be called after 'lvwFindFirstSelItem', returns '0' if no item is seleted
Public Function lvwFindNextSelItem(ByVal lvhWnd As Long) As Long
  lastSelItem = SendMessageA(lvhWnd, LVM_GETNEXTITEM, lastSelItem, ByVal LVIS_SELECTED)
  lvwFindNextSelItem = lastSelItem + 1
End Function

' returns the rectagle coordinates of item 'idx' depending on LVIR_FLAG
Public Function lvwGetItemRect(ByVal lvhWnd As Long, ByVal idx As Long, _
                               Optional ByVal lvir As LVIR_FLAG = LVIR_LABEL) As RECT
  lvwGetItemRect.Left = lvir
  SendMessageA lvhWnd, LVM_GETITEMRECT, idx - 1, lvwGetItemRect
End Function

' returns the item's text in unicode format, if possible:
' idx = ListItem index from 1 to ListItem.Count
Public Function lvwGetItemText(ByVal lvhWnd As Long, ByVal idx As Long, Optional ByVal subItem As Long = 0) As String
  If SendMessageA(lvhWnd, CCM_GETUNICODEFORMAT, 0, ByVal 0) Then
    lvwGetItemText = pGetItemTextW(lvhWnd, idx - 1, subItem)
  Else
    lvwGetItemText = pGetItemTextA(lvhWnd, idx - 1, subItem)
  End If
End Function

' returns the value of check boxes for Common Controls 5.0, which is not supported
' as Property in this version of ActiveX-Control.
' Set LVS_EX_CHECKBOXES in function 'lvwSetExtendedStyle' to make check boxes visible.
' idx = ListItem index from 1 to ListItem.Count
Public Function lvwGetItemChecked(ByVal lvhWnd As Long, ByVal idx As Long) As Boolean
  Dim dwState As Long
  
  dwState = SendMessageA(lvhWnd, LVM_GETITEMSTATE, idx - 1, ByVal LVIS_STATEIMAGEMASK) And LVIS_CHECKED
  lvwGetItemChecked = (dwState = LVIS_CHECKED)
End Function

' get the number of all items
Public Function lvwGetItemCount(ByVal lvhWnd As Long)
  lvwGetItemCount = SendMessageA(lvhWnd, LVM_GETITEMCOUNT, 0&, ByVal 0&)
End Function
' determines the spacing width between ListItems
Public Function lvwGetItemWidth(ByVal lvhWnd As Long, Optional ByVal smallIconView As Boolean = True) As Long
  lvwGetItemWidth = (SendMessageA(lvhWnd, LVM_GETITEMSPACING, smallIconView, 0&) And &HFFFF&)
End Function

' determines the spacing height between ListItems
Public Function lvwGetItemHeight(ByVal lvhWnd As Long, Optional ByVal smallIconView As Boolean = True) As Long
  lvwGetItemHeight = (SendMessageA(lvhWnd, LVM_GETITEMSPACING, smallIconView, 0&) \ &H10000)
End Function

' receives the position of the item's upper-left corner, in view coordinates (NOT absolute)!
' returns True on success, otherwise False
Public Function lvwGetItemPosition(ByVal lvhWnd As Long, ByVal idx As Long, ByRef X As Long, ByRef Y As Long) As Boolean
  Dim pt As POINTAPI
  
  If SendMessageA(lvhWnd, LVM_GETITEMPOSITION, idx - 1, ByVal VarPtr(pt)) Then
    X = pt.X: Y = pt.Y
    lvwGetItemPosition = True
  End If
End Function

' in icon view or small-icon view, the current scroll position is defined by the view origin.
' the view origin is the set of coordinates that correspond to the view coordinates (0, 0),
' relative to the visible area of the list view control. This message should be used ONLY in
' icon view or small icon view; it returns an error in list view or report view!
Public Function lvwGetOrigin(ByVal lvhWnd As Long, ByRef ppt As POINTAPI) As Boolean
  lvwGetOrigin = CBool(SendMessageA(lvhWnd, LVM_GETORIGIN, 0, ppt))
End Function

' get the handle of a listviews header control
Public Function lvwHeaderGetHandle(ByVal lvhWnd As Long) As Long
  lvwHeaderGetHandle = SendMessageA(lvhWnd, LVM_GETHEADER, 0, ByVal 0&)
End Function

' Enable or disable ColumnHeader (disable means: no resizing, no moving)
Public Sub lvwHeaderEnabled(ByVal lvhWnd As Long, Optional Enabled As Boolean = True)
  Dim hdrhWnd As Long
  
  hdrhWnd = lvwHeaderGetHandle(lvhWnd)
  If hdrhWnd Then EnableWindow hdrhWnd, Abs(Enabled)
End Sub

' display ColumnHeaders in a flat style and disable its click event
' for Common Controls Version 5.0 und 6.0
Public Sub lvwHeaderFlatView(ByVal lvhWnd As Long)
  Dim hdrhWnd As Long
  Dim lStyle  As Long
 
  hdrhWnd = lvwHeaderGetHandle(lvhWnd)
  If hdrhWnd Then
    lStyle = GetWindowLongA(hdrhWnd, GWL_STYLE)
    lStyle = lStyle Xor HDS_BUTTONS     ' Style ndern
    SetWindowLongA hdrhWnd, GWL_STYLE, lStyle
    SetWindowPos lvhWnd, GetParent(lvhWnd), 0, 0, 0, 0, SWP_FLAGS
  End If
End Sub

' set the system integrated column arrow for sort up/down (only visible in compiled mode!)
' set "iActiveColumn" to "-1" to remove all arrows!
Public Sub lvwHeaderSortArrow(ByRef lvhWnd As Long, ByVal iActiveColumn As Long, ByVal iSortOrder As ListSortOrderConstants)
  Dim i As Long, cols As Long
  Dim lArrow As Long
  Dim hdrhWnd As Long
  Dim lvc As HDITEMA

  hdrhWnd = lvwHeaderGetHandle(lvhWnd)
  If hdrhWnd Then
    lArrow = IIf(iSortOrder = lvwAscending, HDF_SORTUP, HDF_SORTDOWN)
    cols = SendMessageA(hdrhWnd, HDM_GETITEMCOUNT, 0&, ByVal 0&)
    For i = 0 To cols - 1
      lvc.mask = HDI_FORMAT                         ' we want to get format only
      SendMessageA hdrhWnd, HDM_GETITEMA, i, lvc    ' get fromat of header column
      lvc.fmt = lvc.fmt And Not (HDF_SORTDOWN Or HDF_SORTUP)       ' no arrows in format
      If (i = iActiveColumn) Then                   ' if this is column to activate
        lvc.fmt = lvc.fmt Or lArrow                 ' set system arrow (up/down)
      End If
      SendMessageA hdrhWnd, HDM_SETITEMA, i, lvc    ' set the column data
    Next
  End If
End Sub

' sets a unicode supported column text after 2 preconditions:
' 1st: call once "lvwSetUnicodeFormat" or, even better, "lvwAllInit" at your code
' 2nd: call "lvwHeaderGetHandle" to get the "hdrhWnd", needed as first parameter
' iColumn starts at 0
Public Sub lvwHeaderSetText(ByRef hdrhWnd As Long, ByVal iColumn As Long, ByRef hText As String)
  If SendMessageA(hdrhWnd, CCM_GETUNICODEFORMAT, 0&, ByVal 0&) Then
    Dim hdw As HDITEMW
    hdw.mask = HDI_TEXT                               ' modify text only
    hdw.pszText = StrPtr(hText)                       ' text as pointer
    hdw.cchTextMax = Len(hText)                       ' text length
    SendMessageW hdrhWnd, HDM_SETITEMW, iColumn, hdw  ' send text
  Else
    Dim hda As HDITEMA
    hda.mask = HDI_TEXT                               ' modify text only
    hda.pszText = hText                               ' text as pointer
    hda.cchTextMax = Len(hText)                       ' text length
    SendMessageA hdrhWnd, HDM_SETITEMA, iColumn, hda  ' send text
  End If
End Sub

' Begins in-place editing and return the handle of the edit control
Public Function lvwLabelEdit(ByVal lvhWnd As Long, ByVal idx As Long) As Long
  lvwLabelEdit = SendMessageA(lvhWnd, LVM_EDITLABEL, idx - 1, ByVal 0&)
End Function

Public Function lvwGetEditControl(ByVal lvhWnd As Long) As Long
  lvwGetEditControl = SendMessageA(lvhWnd, LVM_GETEDITCONTROL, 0, ByVal 0&)
End Function

' determines all selected items and returns its values in string array 'theItems'.
' retrun value of function is number of found entries
Public Function lvwSelItemsToArray(ByVal lvhWnd As Long, ByRef theItems() As String) As Long
  Dim Pos As Long, nrItems As Long
  Dim itemText As String
  
  Pos = SendMessageA(lvhWnd, LVM_GETNEXTITEM, -1, ByVal LVIS_SELECTED) + 1
  Do While (Pos)
    If nrItems Then itemText = itemText & vbNullChar
    itemText = itemText & lvwGetItemText(lvhWnd, Pos)
    Pos = SendMessageA(lvhWnd, LVM_GETNEXTITEM, Pos - 1, ByVal LVIS_SELECTED) + 1
    nrItems = nrItems + 1
  Loop
  If nrItems Then theItems = Split(itemText, vbNullChar)
  lvwSelItemsToArray = nrItems
End Function

' ListView background image for Common Controls 5.0
' You can either use an image file or a picture object to set the background
' if both (file and picture) are missing, the background will be cleared up!
' For more information about LVM_SETBKIMAGE please visit the Microsoft MSDN.
Public Sub lvwSetBackgroundImage(ByVal lvhWnd As Long, Optional ByRef imageFileName As String = vbNullString, _
                                 Optional ByRef iPic As IPicture = Nothing, Optional ByVal tile As Boolean = True, _
                                 Optional ByVal xOffPercent As Long = 0, Optional ByVal yOffPercent As Long = 0)
  Dim bgi As LVBKIMAGE
  
  With bgi
    ' clear up any other background and watermark:
    .ulFlags = LVBKIF_SOURCE_NONE
    SendMessageA lvhWnd, LVM_SETBKIMAGE, 0, bgi
    .ulFlags = LVBKIF_TYPE_WATERMARK
    SendMessageA lvhWnd, LVM_SETBKIMAGE, 0, bgi
    ' set new background picture:
    If Len(imageFileName) Then
      .pszImage = imageFileName
      .cchImageMax = Len(.pszImage)
      .ulFlags = LVBKIF_SOURCE_URL
    Else
      .hBM = pBitmapHandleFromPicture(iPic)                   ' needs WinXP or higher!
      .ulFlags = LVBKIF_SOURCE_HBITMAP                        ' only visible in compiled exe!
    End If
    If tile Then .ulFlags = .ulFlags Or LVBKIF_STYLE_TILE
    .xOffsetPercent = xOffPercent
    .yOffsetPercent = yOffPercent
  End With
  SendMessageA lvhWnd, LVM_SETBKIMAGE, 0, bgi                 ' set the background
  SendMessageA lvhWnd, LVM_SETTEXTBKCOLOR, 0&, ByVal -1&      ' set text background transparent
End Sub

' create watermark in Listview Control Version 5.0
' only visible in compiled exe!
Public Sub lvwSetWatermark(ByVal lvhWnd As Long, ByRef iPic As IPicture)
  Dim bgi As LVBKIMAGE
  
  ' first clear up any background
  bgi.ulFlags = LVBKIF_SOURCE_NONE
  SendMessageA lvhWnd, LVM_SETBKIMAGE, 0, bgi
  ' then set watermark image:
  bgi.ulFlags = LVBKIF_TYPE_WATERMARK
  bgi.hBM = pBitmapHandleFromPicture(iPic)
  SendMessageA lvhWnd, LVM_SETBKIMAGE, 0, bgi
  SendMessageA lvhWnd, LVM_SETTEXTBKCOLOR, 0&, ByVal -1&
End Sub

' Size the Column width in report view of Common
' Controls Version 5.0 und 6.0, so "lstView As Object"
' colIndex from 1 to ColumnHeaders.Count
' if "cWidth" is missing or -1 the specified column is autosized!
Public Sub lvwSetColumnWidth(ByVal lstView As Object, ByVal colIndex As Long, Optional ByVal cWidth As Long = -1)
  With lstView
    ' check, if ListView is in view mode "Report"
    If .View = lvwReport Then
      If colIndex > 0 And colIndex <= .ColumnHeaders.Count Then
        SendMessageA .hWnd, LVM_SETCOLUMNWIDTH, colIndex - 1, cWidth
      End If
    End If
  End With
End Sub

' ListView ExplorerStyle, only for Common Controls 5.0
' Needs WinXP or newer. (Effect doesn't exist in IDE!)
Public Function lvwSetExplorerStyle(ByVal lvhWnd As Long) As Boolean
  Dim hModule As Long
  
  ' check if uxtheme.dll exist (only once a session for better performance!)
  If m_uxtheme = uxUNKNOWN Then                         ' function called first time
    hModule = LoadLibraryW(StrPtr("uxtheme.dll"))       ' try to load uxtheme.dll
    If hModule Then                                     ' if exist, then
      m_uxtheme = uxOKAY                                ' keep success in memory
      FreeLibrary hModule                               ' and unload the library
    Else
      m_uxtheme = uxNONE                                ' keep info: no theme dll
    End If
  End If
  
  If m_uxtheme = uxOKAY Then                            ' if dll exist (see above)
    lvwSetExplorerStyle = (SetWindowTheme(lvhWnd, _
                           StrPtr("explorer"), 0) = 0)  ' set the explorer style
  End If
End Function

' ListView style extensions, only Common Controls 5.0
' see 'Enum LVW_EX_STYLES' for possibilities (depending on Windows version)
Public Sub lvwSetExtendedStyle(ByVal lvhWnd As Long, ByVal lStyle As LVW_EX_STYLES, ByVal flag As Boolean)
  Dim mask As Long
  
  mask = SendMessageA(lvhWnd, LVM_GETEXTENDEDLISTVIEWSTYLE, 0&, ByVal 0&)
  If flag Then                  ' switch on style
    mask = mask Or lStyle
  Else                          ' switch off
    mask = mask And Not lStyle
  End If
  SendMessageA lvhWnd, LVM_SETEXTENDEDLISTVIEWSTYLE, ByVal 0&, ByVal mask
  lStyle = GetWindowLongA(lvhWnd, GWL_STYLE) Or WS_CLIPCHILDREN
  SetWindowLongA lvhWnd, GWL_STYLE, lStyle
End Sub

' Bind an imagelist to a ListView and make sure the imagelist is shared,
' so it is not distroyed if the ListView itsself is distroyed!
Public Function lvwSetImageList(ByVal lvhWnd As Long, ByVal himl As Long, ByVal iImageList As LVSIL_Flags) As Long
  Dim dwStyle As Long
  
  dwStyle = GetWindowLongA(lvhWnd, GWL_STYLE)
  If ((dwStyle And LVS_SHAREIMAGELISTS) = False) Then
    Call SetWindowLongA(lvhWnd, GWL_STYLE, dwStyle Or LVS_SHAREIMAGELISTS)
  End If
  lvwSetImageList = SendMessageA(lvhWnd, LVM_SETIMAGELIST, iImageList, ByVal himl)
End Function

' returns the number of selected entries as long value
Public Function lvwGetSelCount(ByVal lvhWnd As Long) As Long
  lvwGetSelCount = SendMessageA(lvhWnd, LVM_GETSELECTEDCOUNT, 0&, ByVal 0&)
End Function

' Returns the index of the item that has the focus rectangle or is selected
' index from 1 to ListItems.Count, 0 means: no item found!
Public Function lvwGetSelectedItem(ByVal lvhWnd As Long) As Long
  lvwGetSelectedItem = SendMessageA(lvhWnd, LVM_GETNEXTITEM, -1, ByVal LVIS_SELECTED) + 1
  If lvwGetSelectedItem = 0 Then  ' get focus only if callback mask of LVIS_FOCUSED is enabled
    If (SendMessageA(lvhWnd, LVM_GETCALLBACKMASK, 0&, 0&) And LVIS_FOCUSED) = 0 Then
      lvwGetSelectedItem = SendMessageA(lvhWnd, LVM_GETNEXTITEM, -1, ByVal LVIS_FOCUSED) + 1
    End If
  End If
End Function

' Select an Item and give it the focus rectangle
' idx = index from 1 to ListItems.Count
Public Function lvwSetSelectedItem(ByVal lvhWnd As Long, ByVal idx As Long, _
                                   Optional ByVal blEnsureVisible As Boolean = False) As Boolean
  If blEnsureVisible Then SendMessageA lvhWnd, LVM_ENSUREVISIBLE, idx - 1, ByVal 1&
  lvwSetSelectedItem = lvwSetItemState(lvhWnd, idx, LVIS_FOCUSED Or LVIS_SELECTED)
End Function

' Get / Set Unicode format
Public Function lvwGetUnicodeFormat(ByVal lvhWnd As Long) As Boolean
  lvwGetUnicodeFormat = SendMessageA(lvhWnd, CCM_GETUNICODEFORMAT, 0&, ByVal 0&) <> 0
End Function
Public Sub lvwSetUnicodeFormat(ByVal lvhWnd As Long, ByVal fUnicode As Boolean)
  SendMessageA lvhWnd, CCM_SETUNICODEFORMAT, Abs(fUnicode), ByVal 0&
  SendMessageA lvwHeaderGetHandle(lvhWnd), CCM_SETUNICODEFORMAT, Abs(fUnicode), ByVal 0&
End Sub

' API function to change view mode
Public Function lvwSetView(ByVal lvhWnd As Long, ByVal newView As Long) As Boolean
  Dim lStyle As Long
  
  ' Get the current view sytle
  lStyle = GetWindowLongA(lvhWnd, GWL_STYLE)
  ' Only set the new view if the current view's bits are different.
  If (lStyle And LVS_TYPEMASK) <> newView Then
    lvwSetView = SetWindowLongA(lvhWnd, GWL_STYLE, (lStyle And Not LVS_TYPEMASK) Or newView)
  End If
End Function

' Load column order from registry for Common Controls 5.0 and 6.0
Public Sub lvwLoadColumnOrder(ByVal lstView As Object)
  Dim nr As Long, cols() As Long
  Dim regString As String
  
  With lstView
    nr = .ColumnHeaders.Count
    If nr = 0 Then Exit Sub
    regString = GetSetting(App.Title, "Settings", .parent.name & "." & .name & "_Columns", "0;")
    cols = pStringToLongArray(regString)
    If UBound(cols) > nr Then ReDim Preserve cols(nr)
    pSetColumnOrderArray .hWnd, UBound(cols), cols(0)
  End With
End Sub

' Save column order to registry for Common Controls 5.0 and 6.0
Public Sub lvwSaveColumnOrder(ByVal lstView As Object)
  Dim nr As Long, cols() As Long
  Dim regString As String
  
  With lstView
    nr = .ColumnHeaders.Count
    If nr = 0 Then Exit Sub
    pGetColumnOrderArray .hWnd, nr, cols
    regString = pLongArrayToString(cols)
    SaveSetting App.Title, "Settings", .parent.name & "." & .name & "_Columns", regString
  End With
End Sub

' redraw the specified icons from index to index
Public Sub lvwRedrawItems(ByVal lvhWnd As Long, Optional ByVal fromIdx As Long = 0, Optional ByVal toIdx = -1)
  If toIdx = -1 Then toIdx = SendMessageA(lvhWnd, LVM_GETITEMCOUNT, 0&, ByVal 0&)
  If fromIdx Then fromIdx = fromIdx - 1
  SendMessageA lvhWnd, LVM_REDRAWITEMS, fromIdx, ByVal toIdx
End Sub

' returns the number of visible elements in a ListView
' works only in report or icon view!
Public Function lvwGetCountPerPage(ByVal lvw As Object) As Long
  Dim w As Long, H As Long
  Dim iw As Long, ih As Long
  
  Select Case lvw.View
    Case lvwReport
      lvwGetCountPerPage = SendMessageA(lvw.hWnd, LVM_GETCOUNTPERPAGE, 0&, ByVal 0&)
    Case lvwIcon
      w = SendMessageA(lvw.hWnd, LVM_GETITEMSPACING, 0&, 0&)
      iw = (w And &HFFFF&): ih = (w \ &H10000)
      w = lvw.Width \ Screen.TwipsPerPixelX
      H = lvw.Height \ Screen.TwipsPerPixelY
      lvwGetCountPerPage = (w \ iw) * (H \ ih)
  End Select
End Function

' Select all items of a listview
Public Function lvwSelectAll(ByVal lvhWnd As Long) As Boolean
  lvwSelectAll = lvwSetItemState(lvhWnd, 0, LVIS_SELECTED)
End Function

Public Function lvwInsertItem(ByVal lvhWnd As Long, ByVal idx As Long, ByVal txt As String, _
                              Optional ByVal subItem As Long = 0) As Long
  Dim pitem As LVITEM
  
  With pitem
    .IItem = idx - 1
    .cchTextMax = Len(txt) + 1
    .pszText = txt & vbNullChar
    .iSubItem = subItem
    .iImage = 1
    .mask = LVIF_TEXT Or LVIF_IMAGE 'Or LVIF_IMAGE Or LVIF_PARAM Or LVIF_INDENT
  End With
  
  lvwInsertItem = SendMessageA(lvhWnd, LVM_INSERTITEM, 0, ByVal pitem)
End Function

' Delete a single entry
' lvhWnd = handle of listview
' idx    = index from 1 to ListItem.Count!
Public Function lvwDeleteItem(ByVal lvhWnd As Long, ByVal idx As Long) As Boolean
  lvwDeleteItem = CBool(SendMessageA(lvhWnd, LVM_DELETEITEM, idx - 1, ByVal 0&))
End Function

' Delete all items of a listview
Public Function lvwDeleteAllItems(ByVal lvhWnd As Long) As Boolean
  lvwDeleteAllItems = CBool(SendMessageA(lvhWnd, LVM_DELETEALLITEMS, 0&, ByVal 0&))
End Function

' delete a listviews column from 1 to columns.count (column 0 cannot be deleted!)
Public Function lvwDeleteColumn(ByVal lvhWnd As Long, ByVal colIdx As Long) As Boolean
  If colIdx Then lvwDeleteColumn = CBool(SendMessageA(lvhWnd, LVM_DELETECOLUMN, colIdx, ByVal 0&))
End Function

' search all selected items and deselect them (fast!)
Public Sub lvwDeselectAll(ByVal lvhWnd As Long)
  Dim nr As Long
  Dim lvi As LVITEM
  
  lvi.State = &H0&
  lvi.stateMask = LVIS_SELECTED Or LVIS_FOCUSED
  nr = SendMessageA(lvhWnd, LVM_GETNEXTITEM, -1, ByVal lvi.stateMask)
  Do While nr > -1
    SendMessageA lvhWnd, LVM_SETITEMSTATE, nr, lvi
    nr = SendMessageA(lvhWnd, LVM_GETNEXTITEM, nr, ByVal lvi.stateMask)
  Loop
End Sub

' makes shure item is visible in the actual view of a listview
' idx = index from 1 to ListItem.Count!
Public Function lvwEnsureVisible(ByVal lvhWnd As Long, ByVal idx As Long) As Boolean
  lvwEnsureVisible = CBool(SendMessageA(lvhWnd, LVM_ENSUREVISIBLE, idx - 1, ByVal 1&))
End Function

' retrieves the icon (item) space (separate for icon view and small icon view)
Public Function lvwGetIconSpace(ByVal lvhWnd As Long, Optional ByVal SmallIcons As Boolean = False, _
                Optional ByRef wSpace As Long, Optional ByRef hSpace As Long) As Long
  Dim isp As Long
  
  isp = SendMessageA(lvhWnd, LVM_GETITEMSPACING, SmallIcons, ByVal 0&)
  wSpace = LoWord(isp): hSpace = HiWord(isp)
  lvwGetIconSpace = isp
End Function

' sets the spacing between icons in listview controls that have the LVS_ICON style
' minimum value must be 4 pixles; to reset the icons to the default spacing,
' set the wSpace and hSpace values &HFFFF
Public Function lvwSetIconSpace(ByVal lvhWnd As Long, _
                ByVal wSpace As Long, ByVal hSpace As Long) As Long
  Dim isp As Long
  
  isp = (wSpace And &HFFFF&) Or (hSpace * &H10000)  ' MakeLong
  lvwSetIconSpace = SendMessageA(lvhWnd, LVM_SETICONSPACING, 0&, ByVal isp)
End Function

Public Function lvwSetItem(ByVal lvhWnd As Long, lvi As LVITEM) As Boolean
  lvwSetItem = CBool(SendMessageA(lvhWnd, LVM_SETITEM, 0, lvi))
End Function

' Add an icon to a subitem (1-n) in report view only
' needs "lvwSetExtendedStyle" with option "LVS_EX_SUBITEMIMAGES" before use!
' lvhWnd  = handle of listview
' idx     = ListItem index from 1 to ListItem.Count!
' subItem = SubItem index from 1 to n (make sure subItem column exists!)
' iconNo  = icon number of connected image list from 1 to n
Public Function lvwSetItemIcon(ByVal lvhWnd As Long, ByVal idx As Integer, ByVal subItem As Long, ByVal iconNo As Long) As Boolean
  Dim lvi As LVITEMW
  
  If iconNo < 0 Then iconNo = -2    ' delete icon if value is negativ
  lvi.IItem = idx - 1
  lvi.iSubItem = subItem
  lvi.iImage = iconNo
  lvi.mask = LVIF_IMAGE
  lvwSetItemIcon = CBool(SendMessageA(lvhWnd, LVM_SETITEM, 0, ByVal VarPtr(lvi)))
End Function

' Enable or Disable the ItemState functionallity of a listview
' lvhWnd  = handle of listview
' States  = one or more LVW_ITEM_STATE, the following values are allowed:
'           LVIS_CUT, LVIS_DROPHILITED, LVIS_FOCUSED, LVIS_SELECTED, LVIS_OVERLAYMASK, LVIS_STATEIMAGEMASK
' Enabled = True=Enable, False=Disable
' Example:  lvwSetItemStateMask lvhWnd, LVIS_FOCUSED, False ' This switches of the focus rectangle
Public Function lvwSetItemStateMask(ByVal lvhWnd As Long, ByVal States As LVW_ITEM_STATE, ByVal Enabled As Boolean) As Boolean
  Dim oldState As LVW_ITEM_STATE
  
  If Enabled Then States = SendMessageA(lvhWnd, LVM_GETCALLBACKMASK, 0&, 0&) And Not States
  lvwSetItemStateMask = SendMessageA(lvhWnd, LVM_SETCALLBACKMASK, States, 0)
End Function
' lvhWnd = handle of listview
' idx    = ListItem index from 1 to ListItem.Count (0 applies to all items!)
Public Function lvwSetItemState(ByVal lvhWnd As Long, ByVal idx As Long, ByVal State As LVW_ITEM_STATE) As Boolean
  Dim lvi As LVITEM
  
  lvi.mask = LVIF_STATE
  lvi.State = State
  lvi.stateMask = &HFFFF
  lvwSetItemState = SendMessageA(lvhWnd, LVM_SETITEMSTATE, idx - 1, lvi)
End Function

' lvhWnd = handle of listview
' idx    = ListItem index from 1 to ListItem.Count (0 applies to all items!)
Public Function lvwRemoveItemState(ByVal lvhWnd As Long, ByVal idx As Long, ByVal State As LVW_ITEM_STATE) As Boolean
  Dim lvi As LVITEM
  
  lvi.mask = LVIF_STATE
  lvi.State = Not State
  lvi.stateMask = State
  lvwRemoveItemState = SendMessageA(lvhWnd, LVM_SETITEMSTATE, idx - 1, lvi)
End Function

' sets an overlay image to an existing icon image
' lvhWnd = handle of listview
' idx    = ListItem index from 1 to ListItem.Count!
' olIdx  = overlay index from 1 to 15 (0 = switch off overlay picture)
' Note:  the attached ImageList MUST contain a mask color and the flag 'UseMakColor' = True
Public Function lvwSetOverlayImage(ByVal lvhWnd As Long, ByVal idx As Long, ByVal olIdx As Long) As Boolean
  Dim lvi As LVITEM
  
  lvi.IItem = idx - 1
  lvi.mask = LVIF_IMAGE Or LVIF_TEXT Or LVIF_STATE
  lvi.State = pINDEXTOOVERLAYMASK(olIdx)
  lvi.stateMask = LVIS_OVERLAYMASK
  lvwSetOverlayImage = SendMessageA(lvhWnd, LVM_SETITEMSTATE, idx - 1, lvi)
End Function

' set a single new item text, or subitem text if 'subItem' > 0
' idx  = ListItem index from 1 to ListItem.Count!
Public Function lvwSetItemText(ByVal lvhWnd As Long, ByVal idx As Long, ByRef txt As String, _
                               Optional ByVal subItem As Long = 0) As Boolean
  Dim lvi As LVITEM
  
  lvi.mask = LVIF_TEXT
  lvi.cchTextMax = Len(txt)
  lvi.pszText = txt
  lvi.iSubItem = subItem
  lvwSetItemText = CBool(SendMessageA(lvhWnd, LVM_SETITEMTEXTA, idx - 1, lvi))
End Function
Public Function lvwSetItemTextW(ByVal lvhWnd As Long, ByVal idx As Long, ByVal txt As String, _
                                Optional ByVal subItem As Long = 0) As Boolean
  Dim lvi As LVITEMW
  
  lvi.mask = LVIF_TEXT
  lvi.cchTextMax = Len(txt) + 1
  lvi.pszText = StrPtr(txt & vbNullChar)
  lvi.iSubItem = subItem
  lvwSetItemTextW = CBool(SendMessageW(lvhWnd, LVM_SETITEMTEXTW, idx - 1, ByVal VarPtr(lvi)))
End Function

' special function for date-, number- and multisort: needs special keywords in the
' ColumnHeaders.Tag property "DATE" for correct date sorting, "NUMBER" for correct
' number sorting (e.g. file size), multisort: up to 3 columns
' lstView  : ListView object common controls V5.0 or V6.0
' Column   : 0 = MainItem, 1 to n = SubItem!
' MoveToTop: if 'True', select and ensure visible the first element after sorting
Public Sub lvwSortColumn(ByRef lstView As Object, ByVal Column As Long, _
                         Optional ByVal MoveToTop As Boolean = False, _
                         Optional ByVal SortOrder As ListSortOrderConstants = lvwAscending)
  Dim idx As Long
  Dim NewSub As Long
  Dim keyWord As String
  Dim strText As String
  Dim hlp As String
  Dim lvi As Object
  Dim blUnicode As Boolean
  
  Const fullDateTime As String = "yyyymmddhhmmss"
  Const maxNumSize As Long = 256
  
  With lstView
    lvwHeaderSortArrow .hWnd, Column, SortOrder
    blUnicode = lvwGetUnicodeFormat(.hWnd)        ' do we support unicode ? (Common contr. 5 only)
    If lvwGetItemCount(.hWnd) Then                ' if any item exist...
      LockWindowUpdate GetParent(.hWnd)           ' fix the window (for faster update)
      .SortOrder = SortOrder                      ' set the wanted sort order
      .ColumnHeaders.Add , , "Dummy", 0           ' create a new dummy column with width '0'
      NewSub = .ColumnHeaders.Count - 1           ' get the number of the new column
      keyWord = .ColumnHeaders(Column + 1).Tag    ' sort type by column Tag property
      Select Case keyWord                         ' sort type by keyword
        Case "FILE"                               ' do not use!!! documentation will follow!
          For Each lvi In .ListItems
            'Dummy-Spalte mit sortierfhiger Dateien/Ordnern belegen
            If Asc(lvi.SubItems(5) & "x") = 32 Then hlp = vbLf$ Else hlp = ""
            If blUnicode Then lvi.Text = Replace(lvi.Text, "?", "zz")
            lvi.SubItems(NewSub) = hlp & lvi.Text
          Next
        Case "NUMBER"                             ' sort numbers by value!
          hlp = Space$(maxNumSize)                ' fill the help string with spaces
          For Each lvi In .ListItems
            If blUnicode Then
              strText = pGetItemTextW(.hWnd, lvi.Index, 0)
            Else
              strText = lvi.Text
            End If
            'Dummy-Spalte mit sortierfhiger Zahl belegen
            If Column Then  ' SubItem
              lvi.SubItems(NewSub) = Right$(hlp & lvi.SubItems(Column), maxNumSize) & strText
            Else            ' MainItem
              lvi.SubItems(NewSub) = Right$(hlp & strText, maxNumSize)
            End If
          Next
        Case "DATE"                               ' sort by date and/or time
          For Each lvi In .ListItems
            'Dummy-Spalte mit sortierfhigem Datum belegen
            If Column Then    ' SubItem
              If IsDate(lvi.SubItems(Column)) Then
                lvi.SubItems(NewSub) = format$(CDate(lvi.SubItems(Column)), fullDateTime)
              End If
            Else              ' MainItem
              If IsDate(lvi.Text) Then
                lvi.SubItems(NewSub) = format$(CDate(lvi.Text), fullDateTime)
              End If
            End If
          Next
        Case "FILEDATE"                           ' sort by datetime and also by folder and files
          For Each lvi In .ListItems
            'Dummy-Spalte mit sortierfhigem Datum belegen
            If Column Then    ' SubItem
              If IsDate(lvi.SubItems(Column)) Then
                If Asc(lvi.SubItems(5)) = 32 Then hlp = vbLf$ Else hlp = ""
                lvi.SubItems(NewSub) = hlp & format$(CDate(lvi.SubItems(Column)), fullDateTime)
              End If
            Else              ' MainItem
              If IsDate(lvi.Text) Then
                lvi.SubItems(NewSub) = format$(CDate(lvi.Text), fullDateTime)
              End If
            End If
          Next
        Case Else
          If IsNumeric(keyWord) Then              ' multisort: up to 3 columns
            idx = val(keyWord)
            If Column Then    ' SubItem
              For Each lvi In .ListItems
                If blUnicode Then
                  strText = pGetItemTextW(.hWnd, lvi.Index, 0)
                Else
                  strText = lvi.Text
                End If
                If idx Then   ' idx > 0: sort by column, subitem(idx), main text
                  lvi.SubItems(NewSub) = lvi.SubItems(Column) & lvi.SubItems(idx) & strText
                Else          ' index = 0: sort by column and main text
                  lvi.SubItems(NewSub) = lvi.SubItems(Column) & strText
                End If
              Next
            Else              ' MainItem
              If idx Then     ' idx > 0: sort by text column, subitem(idx)
                For Each lvi In .ListItems
                  lvi.SubItems(NewSub) = lvi.Text & lvi.SubItems(idx)
                Next
              End If
            End If
          Else                                    ' no keyword in Tag property
            NewSub = Column                       ' normal sorting
          End If  ' IsNumeric(keyWord)
      End Select
      .SortKey = NewSub                           ' set sort column
      .Sorted = True                              ' and sort it :-)
      If MoveToTop Then
        lvwSetSelectedItem .hWnd, 1, True
      Else
        idx = lvwGetSelectedItem(.hWnd)
        SendMessageA .hWnd, LVM_ENSUREVISIBLE, idx, ByVal 1&
      End If
      .Sorted = False
      ' remove dummy column
      .ColumnHeaders.Remove .ColumnHeaders.Count
    End If
  End With
  LockWindowUpdate 0&
End Sub

' lvwHitTest returns "Hit-Item" index from 1 to n (0 = no item found)
' lvhWnd = Handle of the ListView
' plvhti = LVHITTESTINFO structur
Public Function lvwHitTest(ByVal lvhWnd As Long, Optional ByRef mx As Single, Optional ByRef my As Single) As Long
  Dim plvhti As LVHITTESTINFO
  
  GetCursorPos plvhti.pt
  ScreenToClient lvhWnd, plvhti.pt      ' calc. coordinates in listview
  mx = plvhti.pt.X: my = plvhti.pt.Y    ' return mouse position in pixles
  lvwHitTest = SendMessageA(lvhWnd, LVM_HITTEST, 0, plvhti) + 1 ' return item index
End Function

' lvwSubItemHitTest returns "Hit-Item"(plvhti.IItem) from 1 to n (0 = no item found)
' plvhti.iSubItem returns the subitem column from 1 to n, even if no item is selected!
' lvhWnd = Handle of the ListView
' plvhti = LVHITTESTINFO structur
Public Function lvwSubItemHitTest(ByVal lvhWnd As Long, ByRef plvhti As LVHITTESTINFO) As Long
  GetCursorPos plvhti.pt
  ScreenToClient lvhWnd, plvhti.pt
  lvwSubItemHitTest = SendMessageA(lvhWnd, LVM_SUBITEMHITTEST, 0, plvhti) + 1
End Function

' API term of ListView.Refresh
Public Function lvwUpdate(ByVal lvhWnd As Long) As Long
  SendMessageA lvhWnd, LVM_UPDATE, 0&, ByVal 0&
End Function

' Raise scroll event: if 'down' is True, then scroll down, else scroll up!
Public Sub lvwVScroll(ByVal lvhWnd As Long, ByVal down As Boolean)
  SendMessageA lvhWnd, WM_VSCROLL, Abs(down), ByVal 0&
End Sub

' ######################################################################################################## '
' # PRIVATE (p) HELPER FUNCTIONS                                                                         # '
' ######################################################################################################## '

Private Function pGetItemTextA(ByVal lvhWnd As Long, ByVal idx As Long, Optional ByVal subItem As Long = 0) As String
  Dim strLng As Long
  Dim lvi As LVITEM
  
  lvi.pszText = Space$(255)
  lvi.cchTextMax = 255
  lvi.iSubItem = subItem
  strLng = SendMessageA(lvhWnd, LVM_GETITEMTEXTA, idx, lvi)
  If strLng Then pGetItemTextA = Left$(lvi.pszText, strLng)
End Function

Private Function pGetItemTextW(ByVal lvhWnd As Long, ByVal idx As Long, Optional ByVal subItem As Long = 0) As String
  Dim strLng As Long
  Dim strTmp As String
  Dim lvi As LVITEMW
  
  strTmp = String$(255, 0)
  lvi.cchTextMax = Len(strTmp)
  lvi.pszText = StrPtr(strTmp)
  lvi.iSubItem = subItem
  strLng = SendMessageW(lvhWnd, LVM_GETITEMTEXTW, idx, lvi)
  If strLng Then pGetItemTextW = Left$(strTmp, strLng)
End Function

' create a copy of an IPicture object in memory and return its handle
Private Function pBitmapHandleFromPicture(ByRef iPic As IPicture) As Long
  Dim hDCDesktop As Long, hBMP As Long
  Dim hDC1 As Long, hBmpOld1 As Long
  Dim hDC2 As Long, hBmpOld2 As Long
  Dim tBMP As BITMAP
   
  If iPic Is Nothing Then Exit Function
  GetObjectA iPic.handle, Len(tBMP), tBMP
  hDCDesktop = CreateDCA("DISPLAY", ByVal 0&, ByVal 0&, ByVal 0&)
  If hDCDesktop Then
    hDC1 = CreateCompatibleDC(hDCDesktop)
    If hDC1 Then
      hBmpOld1 = SelectObject(hDC1, iPic.handle)
      hDC2 = CreateCompatibleDC(hDCDesktop)
      If hDC2 Then
        hBMP = CreateCompatibleBitmap(hDCDesktop, tBMP.bmWidth, tBMP.bmHeight)
        If hBMP Then
          hBmpOld2 = SelectObject(hDC2, hBMP)
          BitBlt hDC2, 0, 0, tBMP.bmWidth, tBMP.bmHeight, hDC1, 0, 0, vbSrcCopy
          SelectObject hDC2, hBmpOld2
          pBitmapHandleFromPicture = hBMP
        End If  ' hBMP
        DeleteDC hDC2
      End If  ' hDC2
      SelectObject hDC1, hBmpOld1
      DeleteDC hDC1
    End If  ' hDC1
    DeleteDC hDCDesktop
  End If  ' hDCDesktop
End Function

' VB version of Microsofts #define INDEXTOOVERLAYMASK(i) ((i) << 8)
' Prepares the index of an overlay mask so that the ImageList_Draw function can use it.
Private Function pINDEXTOOVERLAYMASK(ByVal idx As Long) As Long
  pINDEXTOOVERLAYMASK = idx * (2 ^ 8)
End Function

Private Function pSetColumnOrderArray(ByVal lvhWnd As Long, ByVal iCount As Long, lpiArray As Long) As Boolean
  pSetColumnOrderArray = SendMessageA(lvhWnd, LVM_SETCOLUMNORDERARRAY, ByVal iCount, lpiArray)
End Function

Private Function pGetColumnOrderArray(ByVal lvhWnd As Long, ByVal iCount As Long, ByRef lpiArray() As Long) As Boolean
  ReDim lpiArray(iCount - 1)
  pGetColumnOrderArray = SendMessageA(lvhWnd, LVM_GETCOLUMNORDERARRAY, ByVal iCount, lpiArray(0))
End Function

Private Function pLongArrayToString(ByRef lArray() As Long, Optional delimiter As String = ";") As String
  Dim i As Long
  
  If IsArray(lArray) Then
    For i = LBound(lArray) To UBound(lArray)
      pLongArrayToString = pLongArrayToString & CStr(lArray(i)) & delimiter
    Next
  End If
End Function

Private Function pStringToLongArray(ByRef s As String, Optional delimiter As String = ";") As Long()
  Dim i As Long
  Dim fld() As Long
  Dim fields() As String
  
  fields = Split(s, delimiter)
  ReDim fld(UBound(fields))
  For i = 0 To UBound(fields)
    fld(i) = val(fields(i))
  Next i
  Erase fields
  pStringToLongArray = fld
End Function

Private Sub pSetHeaderItem(ByVal hWnd As Long, ByVal lCol As Long, tHI As HDITEMA)
    Dim tHIW As HDITEMW
    
    ' Copy fields to tHIW structure
    With tHIW
      .cxy = tHI.cxy
      .fmt = tHI.fmt
      .hBM = tHI.hBM
      .iImage = tHI.iImage
      .iOrder = tHI.iOrder
      .mask = tHI.mask
      If ((.mask And HDI_TEXT) = HDI_TEXT) Then
        .cchTextMax = tHI.cchTextMax
        .pszText = StrPtr(tHI.pszText)
      End If
    End With
    ' Send message as Unicode:
    SendMessageW hWnd, HDM_SETITEMW, lCol, tHIW
End Sub

